@@ -1,5 +1,4 @@
-#
-#  Copyright 2007-2019 Timothy C. Bates
+#  Copyright 2007-2020 Timothy C. Bates
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -13,47 +12,53 @@
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
-# How I coded this data from the Boulder example
-#
-# GFF = read.table("~/bin/umx/data/DHBQ_bs.dat", header = TRUE, sep = "\t", as.is = c(T), na.strings = -999)
-# x   = umx_rename(GFF, old = "zyg2"     , replace = "zyg_2grp"); names(x)
-# x   = umx_rename(x  , old = "zyg"      , replace = "zyg_6grp"); names(x)
-# x   = umx_rename(x , grep = "([12bs])$", replace = "_T\\1")   ; names(x)
-# x$sex_T1 = factor(x$sex_T1, levels = 0:1, labels = c("male", "female"))
-# x$sex_T2 = factor(x$sex_T2, levels = 0:1, labels = c("male", "female"))
-# x$sex_Tb = factor(x$sex_Tb, levels = 0:1, labels = c("male", "female"))
-# x$sex_Ts = factor(x$sex_Ts, levels = 0:1, labels = c("male", "female"))
-# x$zyg_6grp = factor(x$zyg_6grp, levels = 1:6, labels = c("MZMM", "DZMM", "MZFF", "DZFF", "DZFM", "DZMF"))
-# GFF$zyg_2grp = factor(GFF$zyg_2grp, levels = 1:2, labels = c("MZ", "DZ"))
-
-# GFF = GFF[, c("zyg_6grp", "zyg_2grp", "divorce", "sex_T1", "age_T1", "gff_T1", "fc_T1", "qol_T1", "hap_T1", "sat_T1", "AD_T1", "SOMA_T1", "SOC_T1", "THOU_T1", "sex_T2", "age_T2", "gff_T2", "fc_T2", "qol_T2", "hap_T2", "sat_T2", "AD_T2", "SOMA_T2", "SOC_T2", "THOU_T2", "sex_Tb", "age_Tb", "gff_Tb", "fc_Tb","qol_Tb", "hap_Tb", "sat_Tb", "AD_Tb","SOMA_Tb","SOC_Tb", "THOU_Tb","sex_Ts", "age_Ts", "gff_Ts", "fc_Ts", "qol_Ts", "hap_Ts", "sat_Ts", "AD_Ts","SOMA_Ts","SOC_Ts", "THOU_Ts")]
-
-# save("GFF", file = "GFF.rda")
-# system(paste("open ",shQuote(getwd(), type = "csh")))
-# update_wordlist get_wordlist(pkg = "~/bin/umx")
-
-# 1. Figure out what things are.
-
-# table(x$sex_Tb) # all 0 so male = 0
-# table(x$sex_Ts) # all 1 so female = 1
-# umx_aggregate(sex_T2 ~ zyg_6grp, data = x)
-# |zyg_6grp    |sex_T2             |
-# |:-----------|:------------------|
-# |1 (n = 448) |male 448; female 0 |
-# |2 (n = 389) |male 389; female 0 |
-# |3 (n = 668) |male 0; female 668 |
-# |4 (n = 484) |male 0; female 484 |
-# |5 (n = 504) |male 0; female 504 |
-# |6 (n = 407) |male 407; female 0 |
-# umx_aggregate(sex_T1 ~ zyg_6grp, data = x)
-# |zyg_6grp    |sex_T1             |
-# |:-----------|:------------------|
-# |1 (n = 457) |male 457; female 0 |
-# |2 (n = 391) |male 391; female 0 |
-# |3 (n = 661) |male 0; female 661 |
-# |4 (n = 478) |male 0; female 478 |
-# |5 (n = 426) |male 426; female 0 |
-# |6 (n = 460) |male 0; female 460 |
+# ==================================================
+# = Twin data for Direction of causation modelling =
+# ==================================================
+#' Twin data for Direction of causation modelling
+#'
+#' A dataset containing indicators for two traits `varA` and `varB`, each measured in MZ and DZ twins.
+#'
+#' It is designed to show off [umxDoC()] testing the hypothesis `varA` causes `varB`, `varB` causes `varA`, both cause each other.
+#' 
+#' * *zygosity* "MZFF", "DZFF", "MZMM", or "DZMM"
+#' * *varA1_T1* Twin one's manifest 1 for varA
+#' * *varA2_T1* Twin one's manifest 2 for varA
+#' * *varA3_T1* Twin one's manifest 3 for varA
+#' * *varB1_T1* Twin one's manifest 1 for varB
+#' * *varB2_T1* Twin one's manifest 2 for varB
+#' * *varB3_T1* Twin one's manifest 3 for varB
+#' * *varA1_T2* Twin two's manifest 1 for varA
+#' * *varA2_T2* Twin two's manifest 2 for varA
+#' * *varA3_T2* Twin two's manifest 3 for varA
+#' * *varB1_T2* Twin two's manifest 1 for varB
+#' * *varB2_T2* Twin two's manifest 2 for varB
+#' * *varB3_T2* Twin two's manifest 3 for varB
+#' 
+#' @docType data
+#' @keywords datasets
+#' @family datasets
+#' @name docData
+#' @seealso - [umxDoC()], [plot.MxModelDoC()], [umxSummary.MxModelDoC()], [umxModify()]
+#' @references - N.A. Gillespie and N.G. Martin (2005). Direction of Causation Models. 
+#' In *Encyclopedia of Statistics in Behavioral Science*, **1**, 496–499. Eds. Brian S. Everitt & David C. Howell
+#' @usage data(docData)
+#' @format A data frame 6 manifests for each of two twins in 1400 families of MZ and DZ twins
+#' @md
+#' @examples
+#' data(docData)
+#' str(docData)
+#' mzData = subset(docData, zygosity %in% c("MZFF", "MZMM"))
+#' dzData = subset(docData, zygosity %in% c("DZFF", "DZMM"))
+#' par(mfrow = c(1, 2))  # 1 rows and 3 columns
+#' plot(varA1_T2 ~varA1_T1, ylim = c(-4, 4), data = mzData, main="MZ")
+#' tmp = round(cor.test(~varA1_T1 + varA1_T2, data = mzData)$estimate, 2)
+#' text(x=-4, y=3, labels = paste0("r = ", tmp))
+#' plot(varA1_T2 ~varA1_T1, ylim = c(-4, 4), data = dzData, main="DZ")
+#' tmp = round(cor.test(~varA1_T1 + varA1_T2, data = dzData)$estimate, 2)
+#' text(x=-4, y=3, labels = paste0("r = ", tmp))
+#' par(mfrow = c(1, 1))  # back to as it was
+NULL
 
 # ===================================
 # = General Family Functioning data =
@@ -78,65 +83,65 @@
 #' Quality of life in general (QLg) was assessed with the 10-step Cantril
 #' Ladder from best- to worst-possible life (Cantril, 1965).
 #' 
-#' \describe{
-#'   \item{zyg_6grp}{Six-level zygosity: MZMM, DZMM, MZFF, DZFF, DZMF, DZFM}
-#'   \item{zyg_2grp}{Two-level zygosity measure: 'MZ', 'DZ'}
-#'   \item{divorce}{Parental divorce status: 0 = No, 1 = Yes}
-#'   \item{sex_T1}{Sex of twin 1: 0 = "male", 1 = "female"}
-#'   \item{age_T1}{Age of twin 1 (years)}
-#'   \item{gff_T1}{General family functioning for twin 1}
-#'   \item{fc_T1}{Family conflict sub-scale of the FES}
-#'   \item{qol_T1}{Quality of life for twin 1}
-#'   \item{hap_T1}{General happiness for twin 1}
-#'   \item{sat_T1}{Satisfaction with life for twin 1}
-#'   \item{AD_T1}{Anxiety and Depression for twin 1}
-#'   \item{SOMA_T1}{Somatic complaints for twin 1}
-#'   \item{SOC_T1}{Social problems for twin 1}
-#'   \item{THOU_T1}{Thought disorder problems for twin 1}
-#'   \item{sex_T2}{Sex of twin 2}
-#'   \item{age_T2}{Age of twin 2}
-#'   \item{gff_T2}{General family functioning for twin 2}
-#'   \item{fc_T2}{Family conflict sub-scale of the FES}
-#'   \item{qol_T2}{Quality of life for twin 2}
-#'   \item{hap_T2}{General happiness for twin 2}
-#'   \item{sat_T2}{Satisfaction with life for twin 2}
-#'   \item{AD_T2}{Anxiety and Depression for twin 2}
-#'   \item{SOMA_T2}{Somatic complaints for twin 2}
-#'   \item{SOC_T2}{Social problems for twin 2}
-#'   \item{THOU_T2}{Thought disorder problems for twin 2}
-#'   \item{sex_Ta}{Sex of sib 1}
-#'   \item{age_Ta}{Age of sib 1}
-#'   \item{gff_Ta}{General family functioning for sib 1}
-#'   \item{fc_Ta}{Family conflict sub-scale of the FES}
-#'   \item{qol_Ta}{Quality of life for sib 1}
-#'   \item{hap_Ta}{General happiness for sib 1}
-#'   \item{sat_Ta}{Satisfaction with life for sib 1}
-#'   \item{AD_Ta}{Anxiety and Depression for sib 1}
-#'   \item{SOMA_Ta}{Somatic complaints for sib 1}
-#'   \item{SOC_Ta}{Social problems for sib 1}
-#'   \item{THOU_Ta}{Thought disorder problems for sib 1}
-#'   \item{sex_Ts}{Sex of sib 2}
-#'   \item{age_Ts}{Age of sib 2}
-#'   \item{gff_Ts}{General family functioning for sib 2}
-#'   \item{fc_Ts}{Family conflict sub-scale of the FES}
-#'   \item{qol_Ts}{Quality of life for sib 2}
-#'   \item{hap_Ts}{General happiness for sib 2}
-#'   \item{sat_Ts}{Satisfaction with life for sib 2}
-#'   \item{AD_Ts}{Anxiety and Depression for sib 2}
-#'   \item{SOMA_Ts}{Somatic complaints for sib 2}
-#'   \item{SOC_Ts}{Social problems for sib 2}
-#'   \item{THOU_Ts}{Thought disorder problems for sib 2}
-#' }
+#'   * *zyg_6grp*: Six-level zygosity: MZMM, DZMM, MZFF, DZFF, DZMF, DZFM
+#'   * *zyg_2grp*: Two-level zygosity measure: 'MZ', 'DZ'
+#'   * *divorce*: Parental divorce status: 0 = No, 1 = Yes
+#'   * *sex_T1*: Sex of twin 1: 0 = "male", 1 = "female"
+#'   * *age_T1*: Age of twin 1 (years)
+#'   * *gff_T1*: General family functioning for twin 1
+#'   * *fc_T1*: Family conflict sub-scale of the FES
+#'   * *qol_T1*: Quality of life for twin 1
+#'   * *hap_T1*: General happiness for twin 1
+#'   * *sat_T1*: Satisfaction with life for twin 1
+#'   * *AD_T1*: Anxiety and Depression for twin 1
+#'   * *SOMA_T1*: Somatic complaints for twin 1
+#'   * *SOC_T1*: Social problems for twin 1
+#'   * *THOU_T1*: Thought disorder problems for twin 1
+#'   * *sex_T2*: Sex of twin 2
+#'   * *age_T2*: Age of twin 2
+#'   * *gff_T2*: General family functioning for twin 2
+#'   * *fc_T2*: Family conflict sub-scale of the FES
+#'   * *qol_T2*: Quality of life for twin 2
+#'   * *hap_T2*: General happiness for twin 2
+#'   * *sat_T2*: Satisfaction with life for twin 2
+#'   * *AD_T2*: Anxiety and Depression for twin 2
+#'   * *SOMA_T2*: Somatic complaints for twin 2
+#'   * *SOC_T2*: Social problems for twin 2
+#'   * *THOU_T2*: Thought disorder problems for twin 2
+#'   * *sex_Ta*: Sex of sib 1
+#'   * *age_Ta*: Age of sib 1
+#'   * *gff_Ta*: General family functioning for sib 1
+#'   * *fc_Ta*: Family conflict sub-scale of the FES
+#'   * *qol_Ta*: Quality of life for sib 1
+#'   * *hap_Ta*: General happiness for sib 1
+#'   * *sat_Ta*: Satisfaction with life for sib 1
+#'   * *AD_Ta*: Anxiety and Depression for sib 1
+#'   * *SOMA_Ta*: Somatic complaints for sib 1
+#'   * *SOC_Ta*: Social problems for sib 1
+#'   * *THOU_Ta*: Thought disorder problems for sib 1
+#'   * *sex_Ts*: Sex of sib 2
+#'   * *age_Ts*: Age of sib 2
+#'   * *gff_Ts*: General family functioning for sib 2
+#'   * *fc_Ts*: Family conflict sub-scale of the FES
+#'   * *qol_Ts*: Quality of life for sib 2
+#'   * *hap_Ts*: General happiness for sib 2
+#'   * *sat_Ts*: Satisfaction with life for sib 2
+#'   * *AD_Ts*: Anxiety and Depression for sib 2
+#'   * *SOMA_Ts*: Somatic complaints for sib 2
+#'   * *SOC_Ts*: Social problems for sib 2
+#'   * *THOU_Ts*: Thought disorder problems for sib 2
+#'
 #' @docType data
 #' @keywords datasets
 #' @family datasets
 #' @name GFF
 #' @usage data(GFF)
-#' @format A data frame with 1000 rows and 8 variables:
+#' @format A data frame with 1000 rows of twin-family data columns.
 #' @references van der Aa, N., Boomsma, D. I., Rebollo-Mesa, I., Hudziak, J. J., & Bartels, 
 #' M. (2010). Moderation of genetic factors by parental divorce in adolescents' 
 #' evaluations of family functioning and subjective wellbeing. Twin Research 
 #' and Human Genetics, 13(2), 143-162. doi:10.1375/twin.13.2.143
+#' @md
 #' @examples
 #' # Twin 1 variables (end in '_T1')
 #' data(GFF)
@@ -161,26 +166,26 @@ NULL
 #' Calf (fold on the calf muscle), Subscapular (fold over the shoulder blade), 
 #' Suprailiacal (fold between the hip and ribs).
 #' 
-#' * fan FamilyID (t1=male,t2=female)
-#' * zyg Zygosity  1:mzm, 2:mzf, 3:dzm, 4:dzf, 5:dzo
-#' * ht_T1  Height of twin 1 (cm)
-#' * wt_T1  Weight of twin 1 (kg)
-#' * bmi_T1 BMI of twin 1
-#' * bml_T1 log BMI of twin 1
-#' * bic_T1 Biceps Skinfold of twin 1
-#' * caf_T1 Calf Skinfold of twin 1
-#' * ssc_T1 Subscapular Skinfold of twin 1
-#' * sil_T1 Suprailiacal Skinfold of twin 1
-#' * tri_T1 Triceps Skinfold of twin 1
-#' * ht_T2  Height of twin 2
-#' * wt_T2  Weight of twin 2
-#' * bmi_T2 BMI of twin 2
-#' * bml_T2 log BMI of twin 2
-#' * bic_T2 Biceps Skinfold of twin 2
-#' * caf_T2 Calf Skinfold of twin 2
-#' * ssc_T2 Subscapular Skinfold of twin 2
-#' * sil_T2 Suprailiacal Skinfold of twin 2
-#' * tri_T2 Triceps Skinfold of twin 2
+#' * *fan* FamilyID (t1=male,t2=female)
+#' * *zyg* Zygosity  1:mzm, 2:mzf, 3:dzm, 4:dzf, 5:dzo
+#' * *ht_T1*  Height of twin 1 (cm)
+#' * *wt_T1*  Weight of twin 1 (kg)
+#' * *bmi_T1* BMI of twin 1
+#' * *bml_T1* log BMI of twin 1
+#' * *bic_T1* Biceps Skinfold of twin 1
+#' * *caf_T1* Calf Skinfold of twin 1
+#' * *ssc_T1* Subscapular Skinfold of twin 1
+#' * *sil_T1* Suprailiacal Skinfold of twin 1
+#' * *tri_T1* Triceps Skinfold of twin 1
+#' * *ht_T2*  Height of twin 2
+#' * *wt_T2*  Weight of twin 2
+#' * *bmi_T2* BMI of twin 2
+#' * *bml_T2* log BMI of twin 2
+#' * *bic_T2* Biceps Skinfold of twin 2
+#' * *caf_T2* Calf Skinfold of twin 2
+#' * *ssc_T2* Subscapular Skinfold of twin 2
+#' * *sil_T2* Suprailiacal Skinfold of twin 2
+#' * *tri_T2* Triceps Skinfold of twin 2
 #' 
 #' @docType data
 #' @keywords datasets
@@ -192,7 +197,7 @@ NULL
 #' *Archives of Pediatrics and Adolescent Medicine*, **153**, 446-453.
 #' <https://www.ncbi.nlm.nih.gov/pubmed/10323623>
 #' @usage data(us_skinfold_data)
-#' @format A data frame with 53940 rows and 10 variables
+#' @format A data frame with 53940 twin families (1 per row) each twin measured on 10 variables.
 #' @md
 #' @examples
 #' data(us_skinfold_data)
@@ -217,9 +222,11 @@ NULL
 # ==============================
 # = IQ measured longitudinally =
 # ==============================
-#' Twin data: IQ measured longitudinally
+#' Twin data: IQ measured longitudinally across 4 ages.
 #'
-#' Measures of IQ across four ages in 261 pairs of identical twins and 301 pairs of fraternal (DZ) twins. (see details)
+#' Measures of IQ across four ages in 261 pairs of identical twins and 301 pairs of fraternal (DZ) twins. (see details).
+#' It is used as data for the [umxSimplex()] examples.
+#' 
 #' @details
 #' \itemize{
 #'   \item zygosity Zygosity (MZ or DZ)
@@ -237,9 +244,12 @@ NULL
 #' @keywords datasets
 #' @family datasets
 #' @name iqdat
-#' @references TODO
+#' @references Boomsma, D. I., Martin, N. G., & Molenaar, P. C. (1989). Factor and simplex models for repeated measures: 
+#' application to two psychomotor measures of alcohol sensitivity in twins. *Behavior Genetics*, **19**, 79-96.
+#' Retrieved from <https://www.ncbi.nlm.nih.gov/pubmed/2712815>
 #' @usage data(iqdat)
-#' @format A data frame with 562 rows and 9 variables
+#' @format A data frame with 562 rows (twin families). Nine measures on each twin.
+#' @seealso [umxSimplex()] 
 #' @examples
 #' data(iqdat)
 #' str(iqdat)
@@ -283,10 +293,10 @@ NULL
 #' @keywords datasets
 #' @family datasets
 #' @name Fischbein_wt
-#' @references Fischbein, S. (1977). Intra-pair similarity in physical growth of monozygotic and of dizygotic twins during puberty. *Annals of Human Biology*, **4**. 417-430.
-#' \url{https://doi.org/10.1080/03014467700002401}
+#' @references Fischbein, S. (1977). Intra-pair similarity in physical growth of monozygotic and of dizygotic twins during puberty. 
+#' *Annals of Human Biology*, **4**. 417-430. <https://doi.org/10.1080/03014467700002401>
 #' @usage data(Fischbein_wt)
-#' @format A 6*6 correlation matrix based on n = 66 females
+#' @format A 6*6 correlation matrix based on n = 66 female subjects.
 #' @md
 #' @examples
 #' data(Fischbein_wt) # load the data
